"use strict";

var _unistUtilMap = _interopRequireDefault(require("unist-util-map"));

var _textlintUtilToString = _interopRequireDefault(require("textlint-util-to-string"));

var _sentenceSplitter = require("sentence-splitter");

var _splitWords = require("./split-words");

var _objectAssign = _interopRequireDefault(require("object-assign"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// Helper for creating new AST using map function
// https://github.com/azu/unist-util-map
// if you want to filter, use https://github.com/eush77/unist-util-filter
// Helper for converting plain text from Syntax-ed text(markdown AST
// https://github.com/azu/textlint-util-to-string
// Helper for splitting text to sentences
// https://github.com/azu/sentence-splitter
// Helper for splitting text to words
// https://github.com/timjrobinson/split-string-words
// Default options
var defaultOptions = {
  // max count of words >
  max: 50
};
/**
 * @param {TextLintRuleContext} context
 * @param {Object} options
 */

function report(context) {
  var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var {
    Syntax,
    getSource,
    RuleError,
    report
  } = context;
  var maxWordCount = options.max ? options.max : defaultOptions.max;
  return {
    [Syntax.Paragraph](node) {
      // replace code with dummy code
      // if you want to filter(remove) code, use https://github.com/eush77/unist-util-filter
      var filteredNode = (0, _unistUtilMap.default)(node, node => {
        if (node.type === Syntax.Code) {
          // only change `value` to dummy
          return (0, _objectAssign.default)({}, node, {
            value: "code"
          });
        }

        return node;
      });
      var source = new _textlintUtilToString.default(filteredNode); // text in a paragraph

      var text = source.toString(); // get sentences from Paragraph

      var sentences = (0, _sentenceSplitter.split)(text).filter(node => {
        // ignore break line
        return node.type === _sentenceSplitter.Syntax.Sentence;
      }); // text in a sentence

      sentences.forEach(sentence => {
        /* sentence object is a node
        {
            type: "Sentence",
            raw: text,
            value: text,
            loc: loc,
            range: range
        };
         */
        var sentenceText = sentence.value; // words in a sentence

        var words = (0, _splitWords.splitWords)(sentenceText); // over count of word, then report error

        if (words.length > maxWordCount) {
          // get original index value of sentence.loc.start
          var originalIndex = source.originalIndexFromPosition(sentence.loc.start);
          var sentenceFragment = "".concat(words.slice(0, 3).join(' '), " ...");
          var ruleError = new RuleError("Maximum word count (".concat(maxWordCount, ") exceeded (").concat(words.length, ") by \"").concat(sentenceFragment, "\"."), {
            index: originalIndex
          });
          report(node, ruleError);
        }
      });
    }

  };
}

module.exports = report;
//# sourceMappingURL=textlint-rule-en-max-word-count.js.map